@@ -4,6 +4,7 @@
 package wifi
 
 import (
+	"context"
 	"fmt"
 	"runtime"
 	"time"
@@ -21,8 +22,10 @@ func newClient() (*client, error) { return nil, errUnimplemented }
 func (*client) Close() error                                     { return errUnimplemented }
 func (*client) Interfaces() ([]*Interface, error)                { return nil, errUnimplemented }
 func (*client) BSS(_ *Interface) (*BSS, error)                   { return nil, errUnimplemented }
+func (client) AccessPoints(ifi *Interface) ([]*BSS, error)       { return nil, errUnimplemented }
 func (*client) StationInfo(_ *Interface) ([]*StationInfo, error) { return nil, errUnimplemented }
 func (*client) SurveyInfo(_ *Interface) ([]*SurveyInfo, error)   { return nil, errUnimplemented }
+func (*client) Scan(ctx context.Context, ifi *Interface) error   { return errUnimplemented }
 func (*client) Connect(_ *Interface, _ string) error             { return errUnimplemented }
 func (*client) Disconnect(_ *Interface) error                    { return errUnimplemented }
 func (*client) ConnectWPAPSK(_ *Interface, _, _ string) error    { return errUnimplemented }
